@@ -5,9 +5,6 @@ case "$BUILD_MODE" in
     release)
         RUST_LOG=sniffglue target/$TARGET/release/sniffglue -r pcaps/SkypeIRC.pcap > /dev/null
         ;;
-    docker)
-        docker run -e RUST_LOG=sniffglue sniffglue --help
-        ;;
     boxxy)
         if ! cat ci/boxxy_stage0.txt | RUST_LOG=boxxy cargo run --example boxxy | grep -q 'cargo run --example boxxy'; then
             echo SANDOX ERROR: expected match
@@ -30,7 +27,4 @@ case "$BUILD_MODE" in
     cross)
         # do not execute tests when cross compiling
         ;;
-    *)
-        cargo test --verbose --target="$TARGET"
-        ;;
 esac
