@@ -11,7 +11,9 @@ use crate::link::DataLink;
 pub mod arp;
 pub mod tcp;
 pub mod udp;
+pub mod icmp;
 pub mod cjdns;
+pub mod sll;
 
 pub mod dhcp;
 pub mod dns;
@@ -29,6 +31,7 @@ pub fn parse(link: &DataLink, data: &[u8]) -> raw::Raw {
             Err(_)  => Unknown(data.to_vec()),
         },
         DataLink::Tun => parse_tun(data),
+        DataLink::Sll => parse_sll(data),
         DataLink::RadioTap => {
             Unknown(data.to_vec())
         },
@@ -77,6 +80,16 @@ pub fn parse_tun(data: &[u8]) -> raw::Raw {
     )
 }
 
+pub fn parse_sll(data: &[u8]) -> raw::Raw {
+    raw::Raw::Sll(
+        if let Ok(frame) = sll::parse(data) {
+            frame
+        } else {
+            Ether::Unknown(data.to_vec())
+        }
+    )
+}
+
 #[inline]
 pub fn parse_ipv4(data: &[u8]) -> Result<ether::Ether, CentrifugeError> {
     use crate::structs::ipv4::IPv4::*;
@@ -91,6 +104,10 @@ pub fn parse_ipv4(data: &[u8]) -> Result<ether::Ether, CentrifugeError> {
                 Ok((udp_hdr, udp)) => UDP(udp_hdr, udp),
                 Err(_) => Unknown(remaining.to_vec()),
             },
+            IPProtocol::ICMP => match icmp::parse(remaining) {
+                Ok((icmp_hdr, icmp)) => ICMP(icmp_hdr, icmp),
+                Err(_) => Unknown(remaining.to_vec()),
+            },
             _ => {
                 Unknown(remaining.to_vec())
             }
