@@ -1,5 +1,4 @@
-use crate::structs::tcp;
-use crate::structs::udp;
+use crate::structs::{tcp, udp, icmp};
 use crate::structs::NoiseLevel;
 use serde::Serialize;
 
@@ -7,6 +6,7 @@ use serde::Serialize;
 pub enum IPv4 {
     TCP(pktparse::tcp::TcpHeader, tcp::TCP),
     UDP(pktparse::udp::UdpHeader, udp::UDP),
+    ICMP(pktparse::icmp::IcmpHeader, icmp::ICMP),
     Unknown(Vec<u8>),
 }
 
@@ -16,6 +16,7 @@ impl IPv4 {
         match *self {
             TCP(ref header, ref tcp) => tcp.noise_level(header),
             UDP(_, ref udp) => udp.noise_level(),
+            ICMP(ref header, ref icmp) => icmp.noise_level(header),
             Unknown(_) => NoiseLevel::Maximum,
         }
     }
