@@ -12,11 +12,11 @@ pub mod prelude {
     pub use crate::structs::ether::Ether::*;
 }
 
-/// Zero            - This packet is very interesting
-/// One             - This packet is somewhat interesting
-/// Two             - Stuff you want to see if you're looking really hard
-/// AlmostMaximum   - Some binary data
-/// Maximum         - We couldn't parse this
+/// `Zero`            - This packet is very interesting
+/// `One`             - This packet is somewhat interesting
+/// `Two`             - Stuff you want to see if you're looking really hard
+/// `AlmostMaximum`   - Some binary data
+/// `Maximum`         - We couldn't parse this
 #[derive(Debug)]
 pub enum NoiseLevel {
     Zero          = 0,
@@ -36,6 +36,7 @@ pub mod raw;
 pub mod ether;
 pub mod arp;
 pub mod cjdns;
+pub mod icmp;
 pub mod ipv4;
 pub mod ipv6;
 pub mod ip;
